@@ -57,7 +57,7 @@ renv_settings_decode <- function(name, value) {
 }
 
 renv_settings_read <- function(path) {
-  renv_filebacked("settings", path, renv_settings_read_impl)
+  filebacked("settings", path, renv_settings_read_impl)
 }
 
 renv_settings_read_impl <- function(path) {
@@ -100,6 +100,11 @@ renv_settings_get <- function(project, name = NULL) {
     return(settings[order(names(settings))])
   }
 
+  # check for an override via option
+  override <- renv_options_override("renv.settings", name)
+  if (!is.null(override))
+    return(override)
+
   # try to read settings file
   path <- renv_settings_path(project)
   settings <- renv_settings_read(path)
@@ -245,6 +250,16 @@ renv_settings_impl <- function(name, validate, default, update) {
 #'
 #' \describe{
 #'
+#' \item{\code{bioconductor.version}}{
+#'
+#'   The Bioconductor version to be used with this project. Use this if you'd
+#'   like to lock the version of Bioconductor used on a per-project basis.
+#'   When unset, `renv` will try to infer the appropriate Bioconductor release
+#'   using the `BiocVersion` package if installed; if not, `renv` uses
+#'   `BiocManager::version()` to infer the appropriate Bioconductor version.
+#'
+#' }
+#'
 #' \item{\code{external.libraries}}{
 #'
 #'   A vector of library paths, to be used in addition to the project's own
@@ -294,14 +309,22 @@ renv_settings_impl <- function(name, validate, default, update) {
 #'
 #' \item{\code{use.cache}}{
 #'
-#'   Use a global cache of \R packages. When active, `renv` will install
-#'   packages into a global cache, and link packages from the cache into your
-#'   `renv` projects as appropriate. This can greatly save on disk space
+#'   Enable the `renv` package cache with this project. When active, `renv` will
+#'   install packages into a global cache, and link packages from the cache into
+#'   your `renv` projects as appropriate. This can greatly save on disk space
 #'   and install time when for \R packages which are used across multiple
 #'   projects in the same environment.
 #'
 #' }
 #'
+#' \item{\code{vcs.ignore.local}}{
+#'
+#'   Set whether packages within a project-local package cellar are excluded
+#'   from version control. See `vignette("cellar", package = "renv")` for
+#'   more information.
+#'
+#' }
+#'
 #' \item{\code{vcs.ignore.library}}{
 #'
 #'   Set whether the `renv` project library is excluded from version control.
@@ -310,7 +333,8 @@ renv_settings_impl <- function(name, validate, default, update) {
 #'
 #' \item{\code{vcs.ignore.local}}{
 #'
-#'   Set whether `renv` project-specific local sources are excluded from version control.
+#'   Set whether `renv` project-specific local sources are excluded from version
+#'   control.
 #'
 #' }
 #'
@@ -347,6 +371,13 @@ renv_settings_impl <- function(name, validate, default, update) {
 #' }
 settings <- list(
 
+  bioconductor.version = renv_settings_impl(
+    name     = "bioconductor.version",
+    validate = is.character,
+    default  = character(),
+    update   = NULL
+  ),
+
   ignored.packages = renv_settings_impl(
     name     = "ignored.packages",
     validate = is.character,
@@ -389,6 +420,13 @@ settings <- list(
     update   = renv_settings_updated_cache
   ),
 
+  vcs.ignore.cellar = renv_settings_impl(
+    name     = "vcs.ignore.cellar",
+    validate = is.logical,
+    default  = TRUE,
+    update   = renv_settings_updated_ignore
+  ),
+
   vcs.ignore.library = renv_settings_impl(
     name     = "vcs.ignore.library",
     validate = is.logical,
