@@ -34,7 +34,7 @@ test_that("packages can be installed, restored from Bioconductor", {
   expect_true("Biobase" %in% names(records))
 
   if (!renv_platform_linux())
-    renv_scope_options(pkgType = "both")
+    renv_scope_options(pkgType = .Platform$pkgType)
 
   remove("Biobase")
   restore()
@@ -61,3 +61,35 @@ test_that("renv::install(<bioc>, rebuild = TRUE) works", {
   expect_true(renv_package_installed("Biobase"))
 
 })
+
+test_that("bioconductor.version can be used to freeze version", {
+
+  skip_on_cran()
+  project <- renv_tests_scope()
+
+  settings$bioconductor.version("3.14", project = project)
+  expect_equal(renv_bioconductor_version(project = project), "3.14")
+
+})
+
+test_that("we can restore a lockfile using multiple Bioconductor releases", {
+
+  skip_on_cran()
+  skip_on_windows()
+  skip_if(getRversion()[1, 1:2] != "4.1")
+
+  project <- renv_tests_scope()
+
+  path <- file.path(renv_tests_root(), "resources/bioconductor.lock")
+  lockfile <- renv_lockfile_read(path)
+
+  restore(
+    lockfile = lockfile,
+    packages = c("limma", "BiocGenerics"),
+    rebuild  = TRUE
+  )
+
+  expect_true(renv_package_version("limma") == "3.50.0")
+  expect_true(renv_package_version("BiocGenerics") == "0.38.0")
+
+})
